#!/usr/bin/python2

import json
import time
import pycurl
import socket
import argparse
import sys
from dmliteshell.infoutils import SystemInfo

# The top level object
class storageService:

  def __init__(self, config):
    self.config=config
    self.entry={}
    info=SystemInfo(self.config)
    self.entry["name"]=self.config.host
    self.entry["implementation"]="DPM"
    self.entry["implementationversion"]=info.getsysinfo("dome")
    self.entry["qualitylevel"]="production"
    self.entry["storageshares"]=[]
    self.entry["storageendpoints"]=[]
    self.entry["datastores"]=[]
    self.entry["latestupdate"]=int(time.time())

  def addshares(self):
    info=SystemInfo(self.config)
    jgqt=info.getspaces()
    for space in jgqt.keys():
      self.entry["storageshares"].append(storageShare(jgqt[space]["quotatkname"],max(0,jgqt[space]["quotatktotspace"]), max(0,jgqt[space]["pathusedspace"]), jgqt[space]["path"], jgqt[space]["groups"]))

  def addendpoints(self):
    self.entry["storageendpoints"].append(storageEndpoint("gsiftp", "gsiftp://"+self.config.host+"/", "gsiftp"))
    self.entry["storageendpoints"].append(storageEndpoint("https", "https://"+self.config.host+"/", "https"))
    self.entry["storageendpoints"].append(storageEndpoint("xrootd", "root://"+self.config.host+"/", "xrootd"))
    self.entry["storageendpoints"].append(storageEndpoint("srm", "srm://"+self.config.host+":8446/srm/managerv2?SFN=/", "srm"))

  def adddataStores(self):
    self.entry["datastores"].append(dataStore("DPM data store"))

  def printjson(self):
    print json.dumps({"storageservice" : self.entry}, indent=4)

  def publish(self):
    c=pycurl.Curl()
    c.setopt(c.SSLCERT, self.config.cert)
    c.setopt(c.SSLKEY, self.config.key)
    c.setopt(c.SSL_VERIFYPEER, 0)
    c.setopt(c.SSL_VERIFYHOST, 2)
    c.setopt(c.FOLLOWLOCATION, 1)

    try:
      # check path exists
      # here it is
      if (self.config.davport != 443) :
        self.config.host = self.config.host + ":" + str(self.config.davport)
      c.setopt(c.URL, "https://"+self.config.host+"/"+self.config.path)
      c.setopt(c.NOBODY, True)
      c.setopt(c.CUSTOMREQUEST, "HEAD")
      c.perform()
      if (c.getinfo(c.HTTP_CODE) != 200):
        print("Path "+self.config.path+" not found")
        sys.exit(1)
        
      # delete existing file
      c.setopt(c.URL, "https://"+self.config.host+"/"+self.config.path+"/storagesummary.json")
      c.setopt(c.NOBODY, True)
      c.setopt(c.CUSTOMREQUEST, "DELETE")
      c.perform()
      if (c.getinfo(c.HTTP_CODE) != 204 and c.getinfo(c.HTTP_CODE) != 404):
        print("Can't delete https://"+self.config.host+"/"+self.config.path+"/storagesummary.json")
        sys.exit(1)

      # put the new file
      c.setopt(c.NOBODY, False)
      c.setopt(c.CUSTOMREQUEST, "PUT")
      # suppress the response body
      c.setopt(c.WRITEFUNCTION, lambda x: None)
      c.setopt(c.POSTFIELDS, json.dumps({"storageservice" : self.entry}, indent=4))
      c.perform()
      if (c.getinfo(c.HTTP_CODE) == 201):
        print("https://"+self.config.host+"/"+self.config.path+"/storagesummary.json"+" written successfully")
      else:
        #self.printjson()
        print("ERROR: Put new summary file : https://"+self.config.host+"/"+self.config.path+"/storagesummary.json"+" NOT written successfully")
        print("Return http_code="+str(c.getinfo(c.HTTP_CODE)))
    except Exception as e:
      print("File writing error")
      print(e)
    c.close()

# Derive from dict for easy serialisation    
class storageEndpoint(dict):
  def __init__(self, name, endpointurl, interfacetype):
    self["name"]=name
    self["endpointurl"]=endpointurl
    self["interfacetype"]=interfacetype
    self["qualitylevel"]="production"
    self["assignedshares"]=["all"]

# Derive from dict for easy serialisation    
class storageShare(dict):
  def __init__(self, name, totalsize, usedsize, path, groups):
    self["name"]=name
    self["timestamp"]=int(time.time())
    self["totalsize"]=int(totalsize)
    self["usedsize"]=int(usedsize)
    self["path"]=[path]
    self["vos"]=groups
    self["assignedendpoints"]=["all"]
    self["servingstate"]="open"
    
# Derive from dict for easy serialisation    
class dataStore(dict):
  def __init__(self, name):
    self["name"]=name
    self["message"]="Not used"


# Arguments
parser = argparse.ArgumentParser()
parser.add_argument("--path", help="Path to publish the summary file", default="/dpm/cern.ch/home/dteam")
parser.add_argument("--cert", help="Path to host certificate", default="/etc/grid-security/hostcert.pem")
parser.add_argument("--key", help="Path to host key", default="/etc/grid-security/hostkey.pem")
parser.add_argument("--host", help="FQDN", default=socket.getfqdn())
parser.add_argument("--print", help="Just print, don't publish", action="store_true", dest="prnt")
parser.add_argument("--port", help="Dome's port number", default=1094)
parser.add_argument("--davport", help="Webdav's port if !=443", default=443)

args = parser.parse_args()

# Create object
dpm=storageService(args)
dpm.addshares()

# Which endpoints are supposed to be there?
dpm.addendpoints()
# Don't bother with this for now
#dpm.adddataStores()

# Print or publish
if args.prnt:
  dpm.printjson()
else:
  dpm.publish()
